package it.tref.liferay.mapit.action;


import com.liferay.portal.kernel.events.ActionException;
import com.liferay.portal.kernel.events.SimpleAction;
import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.GetterUtil;
import com.liferay.portal.kernel.util.UnicodeProperties;
import com.liferay.portal.model.Group;
import com.liferay.portal.model.ResourceConstants;
import com.liferay.portal.model.Role;
import com.liferay.portal.model.RoleConstants;
import com.liferay.portal.security.permission.ActionKeys;
import com.liferay.portal.service.ResourcePermissionLocalServiceUtil;
import com.liferay.portal.service.RoleLocalServiceUtil;
import com.liferay.portal.util.PortalUtil;
import com.liferay.portlet.expando.model.ExpandoColumn;
import com.liferay.portlet.expando.model.ExpandoColumnConstants;
import com.liferay.portlet.expando.model.ExpandoTable;
import com.liferay.portlet.expando.model.ExpandoTableConstants;
import com.liferay.portlet.expando.service.ExpandoColumnLocalServiceUtil;
import com.liferay.portlet.expando.service.ExpandoTableLocalServiceUtil;

public class StartUpAction extends SimpleAction {

  private final static Log _log = LogFactoryUtil.getLog(StartUpAction.class);

  @Override
  public void run(String[] companyIds) throws ActionException {

    for (String id : companyIds) {
      try {
        long companyId = GetterUtil.getLong(id);

        String[] defaultActions = {ActionKeys.VIEW, ActionKeys.UPDATE};
        String[] guestActions = {ActionKeys.VIEW};

        // Organization custom fields
        addExpandoColumnIfNotExists(companyId, Group.class.getName(), "portos_mapit_static_js_enable",
            ExpandoColumnConstants.BOOLEAN, false, false, defaultActions, guestActions);
      } catch (Exception e) {
        _log.error("Errore nella procedura automatizzata di configurazione", e);
      }
    }
  }

  private ExpandoColumn addExpandoColumnIfNotExists(long companyId, String className, String name, int type,
      boolean indexed, boolean hidden, String[] defaultActions, String[] guestActions) throws SystemException {

    long classNameId = PortalUtil.getClassNameId(className);

    ExpandoTable expandoTable = null;
    try {
      expandoTable =
          ExpandoTableLocalServiceUtil.getTable(companyId, classNameId, ExpandoTableConstants.DEFAULT_TABLE_NAME);
    } catch (PortalException e) {
      try {
        expandoTable =
            ExpandoTableLocalServiceUtil.addTable(companyId, classNameId, ExpandoTableConstants.DEFAULT_TABLE_NAME);
      } catch (PortalException e1) {
      }
    }

    ExpandoColumn expandoColumn = ExpandoColumnLocalServiceUtil.getColumn(expandoTable.getTableId(), name);

    if (expandoColumn == null) {

      try {
        expandoColumn = ExpandoColumnLocalServiceUtil.addColumn(expandoTable.getTableId(), name, type);

      } catch (PortalException e) {
      }
    }

    try {

      if (indexed) {

        UnicodeProperties typeSettingsProperties = expandoColumn.getTypeSettingsProperties();

        typeSettingsProperties.put(ExpandoColumnConstants.INDEX_TYPE,
            String.valueOf(ExpandoColumnConstants.INDEX_TYPE_TEXT));

        expandoColumn.setTypeSettingsProperties(typeSettingsProperties);

        expandoColumn = ExpandoColumnLocalServiceUtil.updateExpandoColumn(expandoColumn);
      }

      if (hidden) {

        UnicodeProperties typeSettingsProperties = expandoColumn.getTypeSettingsProperties();

        typeSettingsProperties.put(ExpandoColumnConstants.PROPERTY_HIDDEN, String.valueOf(1));

        expandoColumn.setTypeSettingsProperties(typeSettingsProperties);

        expandoColumn = ExpandoColumnLocalServiceUtil.updateExpandoColumn(expandoColumn);
      }

      Role roleGuest = RoleLocalServiceUtil.getRole(companyId, RoleConstants.GUEST);
      Role rolePowerUser = RoleLocalServiceUtil.getRole(companyId, RoleConstants.POWER_USER);
      Role roleUser = RoleLocalServiceUtil.getRole(companyId, RoleConstants.USER);

      ResourcePermissionLocalServiceUtil.setResourcePermissions(companyId, ExpandoColumn.class.getName(),
          ResourceConstants.SCOPE_INDIVIDUAL, String.valueOf(expandoColumn.getColumnId()), roleGuest.getRoleId(),
          guestActions);
      ResourcePermissionLocalServiceUtil.setResourcePermissions(companyId, ExpandoColumn.class.getName(),
          ResourceConstants.SCOPE_INDIVIDUAL, String.valueOf(expandoColumn.getColumnId()), rolePowerUser.getRoleId(),
          defaultActions);
      ResourcePermissionLocalServiceUtil.setResourcePermissions(companyId, ExpandoColumn.class.getName(),
          ResourceConstants.SCOPE_INDIVIDUAL, String.valueOf(expandoColumn.getColumnId()), roleUser.getRoleId(),
          defaultActions);
    } catch (PortalException e) {
    }

    return expandoColumn;
  }

}
