
AUI().applyConfig({
	modules: {
        'open-layers': {
        	fullpath: '/tref-mapit-map-portlet/open_layers/OpenLayers.js',
        }
    }
});

AUI.add(
	'liferay-mapit-map',
	function(A) {
		
		var _cache = {};
		var _map = {};

		var MapItMap = A.Component.create(
			{
				EXTENDS: A.Base,
				
				NAME: 'mapitmap',
				
				ATTRS: {
					id: {
						value: ''
					},
					portletId: {
						value: ''
					},
					mapType: {
						value: 1
					},
					apiKey: {
						value: ''
					},
					apiVersion: {
						value: ''
					},
					mapStyles: {
						value: ''
					},
					center: {
						value: {
							latitude: 0,
							longitude: 0,
							zoom: 1,
							geoLocation: false
						}
					},
					saveLastPosition: {
						value: false
					},
					controls: {
						value: {
							layerSwitch: false,
							fullScreen: false,
							navToolbar: false,
							mousePosition: false,
							scale: false,
							home: false,
							overviewMap: false, 
							zoomType: null
						}
					}
				},
				
				MAP_TYPE: {
					OPEN_STREET_MAP: 1,
					GOOGLE_MAP: 2,
					BING_MAP: 3,
					BASE: 4,
					BASE_LABEL: 5
				},
			
				ZOOM_TYPE: {
					ZOOM: 1,
					PAN_ZOOM: 2,
					PAN_ZOOM_BAR: 3,
				},
				
				DEFAULT_MARKER: '/tref-mapit-map-portlet/open_layers/img/marker.png',
				
				OPEN_LAYERS_LANG_JS: '/tref-mapit-map-portlet/open_layers/lang/',
	
				get: function(portletId) {
	
					var mapItMap = null;
					
					if (_cache[portletId]) {
						mapItMap = _cache[portletId];
					}
					return mapItMap;
				},

				prototype: {
					
					initializer: function(config) {
						var instance = this;
						
						instance.config = config;
						
						var portletId = instance.get('portletId');
						var namespace = Liferay.Util.getPortletNamespace(portletId);
						
						/**
						 * @Override createUniqueId
						 */
						OpenLayers.Util.createUniqueID = function(prefix) {
						    if (prefix == null) {
						        prefix = "id_";
						    } else {
						        prefix = prefix.replace(OpenLayers.Util.dotless, "_");
						    }
						    OpenLayers.Util.lastSeqID += 1; 
						    return namespace + prefix + OpenLayers.Util.lastSeqID; 
						};
					},

					render: function() {
						var instance = this;
						
						var language = themeDisplay.getLanguageId().split('_');
						
						var languageJs = MapItMap.OPEN_LAYERS_LANG_JS + language[0] + '.js';
		
						A.Get.script(
							languageJs, {
								onEnd: function(event) {
									
									if (!event.errors) {
										OpenLayers.Lang.setCode(language[0]);
									}
									
									var id = instance.get('id');
									
									var portletId = instance.get('portletId');
									
									var map = new OpenLayers.Map({
									    div: id,
									    projection: 'EPSG:4326',
									    numZoomLevels: 21,
									    controls: [],
									    fractionalZoom: false,
									    lastSeqLayerID: 0,
									    lastSeqPopupId: 0,
									});
									
									instance.mapId = map.id;
									
									_map[instance.mapId] = map;
									
									if (!!portletId) {
										_cache[portletId] = instance;
									}
									
									if (instance.get('mapType') == MapItMap.MAP_TYPE.GOOGLE_MAP) {
										if (parseFloat(instance.get('center.latitude')) < 0.09
												&& parseFloat(instance.get('center.latitude')) > -0.09) {
											instance.set('center.latitude', '0.09');
										}
										if (parseFloat(instance.get('center.longitude')) < 0.09
												&& parseFloat(instance.get('center.longitude')) > -0.09) {
											instance.set('center.longitude', '0.09');
										}
									}
									
									// default events
									instance.publish('featureSelected');
									instance.publish('featureUnselected');
									instance.publish('layerReady');
									instance.publish('ready');
								
									instance._moveEvents();
									instance._loadMap();
								}
							}
						);
				    },
				    
				    addLayerPoints: function(layerName, points, clear, options) {
				    	var instance = this;		
				    	
						var map = _map[instance.mapId];

						var layer = null;
						
						var layers = map.getLayersByName(layerName);
				    	if (layers.length > 0){
				    		layer = layers[0];
				    		if (clear) {
				    			layer.removeAllFeatures();
				    		} 
				    	} else {
				    		layer = new OpenLayers.Layer.Vector(layerName, {
				    			stategies: new OpenLayers.Strategy.Fixed(),
				    			mapItId: (++map.lastSeqLayerID),
				    			lastSeqID: 0
				    		});
				    						    		
				    		instance._addLayer(layer, options);
				    	}
				    	
				    	var features = [];
				    	for (var i = points.length - 1; i >= 0; --i) {
						    var point = points[i];
						    
						    var geometry = new OpenLayers.Geometry.Point(
						    	parseFloat(point.lon), parseFloat(point.lat)).transform(
								new OpenLayers.Projection("EPSG:4326"), map.getProjectionObject()
							);
						    
						    var attribute = {
								title: point.title,
								description: point.description
							}; 
						 
						    var style = null;
						    
						    if (point.style){
						    	style = point.style;
						    }
						    else if (point.icon) {					    	
						    	style = {
					    			externalGraphic: point.icon, 
									graphicHeight: 25, 
									graphicWidth: 21, 
									graphicXOffset:-12, 
									graphicYOffset:-25,
									cursor: 'pointer'
								};
						    }
						    else {
						    	style = null;
						    }
						   
						    var feature = new OpenLayers.Feature.Vector(geometry, attribute, style); 
						    feature.mapItId = (++layer.lastSeqID);
						    
						    features.push(feature);
						}
				    	
				    	layer.addFeatures(features);
				    },
				    /** Added for Portos (start) **/
				    addInvisibleLayerPoints: function(layerName, points, clear, options) {
				    	var instance = this;		
				    	
						var map = _map[instance.mapId];

						var layer = null;
						
						console.log(options)
						
						var layers = map.getLayersByName(layerName);
				    	if (layers.length > 0){
				    		layer = layers[0];
				    		if (clear) {
				    			layer.removeAllFeatures();
				    		} 
				    	} else {
				    		layer = new OpenLayers.Layer.Vector(layerName, {
				    			stategies: new OpenLayers.Strategy.Fixed(),
				    			mapItId: (++map.lastSeqLayerID),
				    			lastSeqID: 0,
				    			displayInLayerSwitcher: false
				    		});
				    						    		
				    		instance._addLayer(layer, options);
				    	}
				    	
				    	var features = [];
				    	for (var i = points.length - 1; i >= 0; --i) {
						    var point = points[i];
						    
						    var geometry = new OpenLayers.Geometry.Point(
						    	parseFloat(point.lon), parseFloat(point.lat)).transform(
								new OpenLayers.Projection("EPSG:4326"), map.getProjectionObject()
							);
						    
						    var attribute = {
								title: point.title,
								description: point.description
							}; 
						 
						    var style = null;
						    
						    if (point.style){
						    	style = point.style;
						    }
						    else if (point.icon) {					    	
						    	style = {
					    			externalGraphic: point.icon, 
									graphicHeight: 25, 
									graphicWidth: 21, 
									graphicXOffset:-12, 
									graphicYOffset:-25,
									cursor: 'pointer'
								};
						    }
						    else {
						    	style = null;
						    }
						   
						    var feature = new OpenLayers.Feature.Vector(geometry, attribute, style); 
						    feature.mapItId = (++layer.lastSeqID);
						    
						    features.push(feature);
						}
				    	
				    	layer.addFeatures(features);
				    },
				    addWMSLayer: function(layer){
				    	var instance = this;
				    	var map = _map[instance.mapId];
				    	
				    	map.addLayer(layer);				    	
				    },
				    
				    addBasicLayer: function(layer, options){
				    	var instance = this;						    	
				    	
				    	instance._addLayer(layer, options);
				    },
				    
				    addOption: function(options){
				    	var instance = this;
				    	var map = _map[instance.mapId];
				    	
				    	map.setOptions(options);
				    },
				    
				    getInstance: function(){
				    	var instance = this;
				    	var map = _map[instance.mapId];
				    	
				    	return map;
				    },
				    
				    getPositionOnClick: function(event){
				    	var instance = this;
				    	var map = _map[instance.mapId];
				    	
				    	return map.getLonLatFromPixel(event.xy);
				    },
				    
				    registerFunctionOnEvent: function(event, func){
				    	var instance = this;
				    	var map = _map[instance.mapId];
				    	
				    	map.events.register("click", map, func );
				    },
				    
				    insertControl: function(control){
				    	var instance = this;
				    	var map = _map[instance.mapId];
				    	
				    	map.addControl(control);
				    },
				    /** Added for Portos (end) **/
				    addLayer: function(layerName, url, params, options) {
				    	var instance = this;
				    
				    	var map = _map[instance.mapId];
				    	
				    	var fixed = false;
				    	var zoomToAllFeatures = false;
				    	
				    	if (options) {
				    		zoomToAllFeatures = (typeof options.zoomToAllFeatures != 'undefined' && options.zoomToAllFeatures);
				    		fixed = (typeof options.fixed != 'undefined' && options.fixed);
				    	}
				    	
				    	var stategies = [];
				    	if (zoomToAllFeatures || fixed) {	
				    		stategies.push(new OpenLayers.Strategy.Fixed());
				    	}
				    	else {
				    		stategies.push(new OpenLayers.Strategy.BBOX({ratio: 1, resFactor: 1.1}));
				    	}

						var layer = new OpenLayers.Layer.Vector(layerName, {
							strategies: stategies,
							protocol: new OpenLayers.Protocol.HTTP({
								url: url,
								params: params,
								readWithPOST: true,
								format: new OpenLayers.Format.MapItJSON()
							}),
							mapItId: (++map.lastSeqLayerID)
						});
						
						instance._addLayer(layer, options);				
					},
					
					moveMap: function(longitude, latitude, zoom) {
						var instance = this;
						
						var map = _map[instance.mapId];
						
						var lonlatObject = new OpenLayers.LonLat(longitude, latitude).transform(
							new OpenLayers.Projection('EPSG:4326'), map.getProjectionObject()
						);
						
					    map.moveTo(lonlatObject, zoom);
					},
					
					scrollIntoView: function() {
						var instance = this;
						
						A.one('#' + instance.get('id')).scrollIntoView();
					},
					
					showPopup: function(options) {
						var instance = this;
						
						var map = _map[instance.mapId];
						
						var popupId = (++map.lastSeqPopupId);
	
						var feature = null;
						if (!!options.layerId && !!options.featureId) {
							var layers = map.getLayersBy('mapItId', options.layerId);
							if (layers.length > 0) {
								feature = layers[0].getFeatureBy('mapItId', options.featureId);
							}
						}
						
						var latitude = options.latitude;
						var longitude = options.longitude;
						
						var width = 50;
						if (typeof options.width != 'undefined') {
							width = options.width;
						}
						var height = 50;
						if (typeof options.height != 'undefined') {
							height = options.height;
						} 
						
						var lonlatObject = new OpenLayers.LonLat(longitude, latitude).transform(
							new OpenLayers.Projection('EPSG:4326'), map.getProjectionObject());
						
						var contentHtml = '';
						if (typeof options.contentHtml != 'undefined') {
							contentHtml = options.contentHtml;
						} 
						else if (typeof options.url != 'undefined') {
							
							options.autoSize = false;

							var url = options.url;
							if (typeof options.parameters != 'undefined') {
								url = Liferay.Util.addParams(options.parameters, options.url);
							}
							
							var portletId = instance.get('portletId');
							var namespace = Liferay.Util.getPortletNamespace(portletId);
							
							var iframeId = (namespace + 'mapit_iframe_popup_' + popupId);
							var loadingId = (namespace + 'mapit_loading_popup_' + popupId);
							var loading = '<img src="' + (themeDisplay.getPathThemeImages() + '/application/loading_indicator.gif') + '" />';
							var script = 'document.getElementById(\'' + loadingId + '\').style.display = \'none\';';
							script += 'AUI().use(\'liferay-mapit-map\', function(A) { var mapItMap = Liferay.MapItMap.get(\'' + portletId + '\'); mapItMap._resizePopup(\'' + iframeId + '\', ' + popupId + ');});';							;
								
							contentHtml = '<div id="' + loadingId + '" style="text-align: center;">' + loading + '</div>';
							contentHtml += '<div class="mapit-popup-content"><iframe id="' + iframeId + '" onload="' + script +'" src="' + url + '" frameborder="0" scrolling="auto" marginheight="0" marginwidth="0" height="100%"></iframe></div>';
						}
	
						var popup = new OpenLayers.Popup.MapItFramedCloud(null, lonlatObject,
							new OpenLayers.Size(width, height), contentHtml, null, true,
				    		function(event) {
								var feature = this.feature;
				                if (feature && feature.layer && feature.layer.mapItType == 'mapItSelectLayer') {
				                	var selectControl = map.getControlsBy('mapItId', 'mapItSelectFeature')[0];
				                	selectControl.unselect(feature);
				                } else { 
				                    instance.hidePopup({
				                    	popupId: this.mapItId
				                    });
				                }
				    		}
					    );						
						popup.mapItId = popupId;

						if (typeof options.closeOnMove != 'undefined') {
							popup.mapItCloseOnMove = options.closeOnMove;
						}
						else {
							popup.mapItCloseOnMove = false;
						}
						
						if (typeof options.autoSize != 'undefined') {
							popup.autoSize = options.autoSize;
						}
						
						if (feature) {
							feature.popup = popup;
							popup.feature = feature;
						}
						
					    map.addPopup(popup, false);
					    
					    return popup.mapItId;
					},
					
					hidePopup: function(options) {
						var instance = this;
						
						var map = _map[instance.mapId];
						
						var featureId = options.featureId;
						var layerId = options.layerId;
						
						if (!!featureId && !!layerId) {
							var layers = map.getLayersBy('mapItId', layerId);
							if (layers.length > 0) {
								var feature = layers[0].getFeatureBy('mapItId', featureId);
								if (feature) {
									if (feature.popup) {
								    	map.removePopup(feature.popup);
								    	feature.popup.destroy();
								        feature.popup = null;
								    }
								}
								else {
									var popups = map.popups;
									for (var i=0; i<popups.length && popups[i]; i++) {
										var popup = popups[i];
										if (popup.feature && popup.feature.mapItId == featureId) {
											map.removePopup(popup);
											popup.destroy();
											popup = null;
											break;
										}
									}
								}
							}
						}
						else if (!!options.popupId) {
							var popupId = options.popupId;
							var popups = map.popups;
							for (var i=0; i<popups.length && popups[i]; i++) {
								var popup = popups[i];
								if (popup.mapItId == popupId) {
									map.removePopup(popup);
									popup.destroy();
									popup = null;
									break;
								}
							}	
						}
					},
					
					getCurrentZoom: function() {
						var instance = this;
						
						return _map[instance.mapId].getZoom();
					},
					
					zoomToExtent: function(bounds, closest){
						var instance = this;	
						
						var map = _map[instance.mapId];
						
						if (Liferay.Util.isArray(bounds)) {
							// [left, bottom, right, top]
							// left, bottom
							var lonlatObject1 = new OpenLayers.LonLat(bounds[0], bounds[1]).transform(
								new OpenLayers.Projection('EPSG:4326'), map.getProjectionObject()
							);
							// right, top
							var lonlatObject2 = new OpenLayers.LonLat(bounds[2], bounds[3]).transform(
								new OpenLayers.Projection('EPSG:4326'), map.getProjectionObject()
							);
							bounds = [lonlatObject1.lon, lonlatObject1.lat, lonlatObject2.lon, lonlatObject2.lat];
						}
						
						map.zoomToExtent(bounds, closest);
					},
					
					zoomToDataExtent: function(layerName){
						var instance = this;
						
						var map = _map[instance.mapId];

						var layers = null;
						if (layerName) {
							layers = map.getLayersByName(layerName);
						}
						
						if (layers && layers.length > 0 && layers[0].features.length > 0) {
				    		var layer = layers[0];
				    		map.zoomToExtent(layer.getDataExtent());
				    	} else {
				    		var bound = null;
				    		var allLayers = map.getLayersByClass(OpenLayers.Layer.Vector.prototype.CLASS_NAME);
				    		for (var i=0; i< allLayers.length; i++) {
				    			if (bound) {
				    				bound.extend(allLayers[i].getDataExtent());
				    			} else {
				    				bound = allLayers[i].getDataExtent();
				    			}
				    		}
				    		
				    		if (bound) {
				    			map.zoomToExtent(bound);
				    		}
				    	}
					},
					
					attachDrag: function(layerName, options) {
						var instance = this;
						
						var map = _map[instance.mapId];
						
						var layers = map.getLayersByName(layerName);
						if (layers.length > 0) {
							var controls = map.getControlsByClass(OpenLayers.Control.DragFeature.prototype.CLASS_NAME)
							var attached = false;
							for (var i=0; i<controls.length && !attached; i++) {
								if (controls[i] && controls[i].layer && controls[i].layer.name == layerName) {
									attached = true;
								}
							}
							
							if (!attached) {
								
								if (options && options.on && !(typeof options.on.scope != 'undefined')) {
									options.on.scope = window;
								}
								
								var control = new OpenLayers.Control.DragFeature(layers[0], {
									onStart: function(event) {
										if (options.on && options.on.start) {
											var map = event.layer.map;
											var lonLat = event.geometry.getBounds().getCenterLonLat();
											var lonlatObject = new OpenLayers.LonLat(lonLat.lon, lonLat.lat).transform(
												map.getProjectionObject(), new OpenLayers.Projection('EPSG:4326'));
											options.on.start.call(options.on.scope, {
												layer: {
													id: event.layer.mapItId,
													name: event.layer.name
												},
												id: event.mapItId,
												attributes: event.attributes,
												latitude: lonlatObject.lat,
												longitude: lonlatObject.lon
											});
										}
									},
									onComplete: function(event) {
										if (options.on && options.on.complete) {
											var map = event.layer.map;
											var lonLat = event.geometry.getBounds().getCenterLonLat();
											var lonlatObject = new OpenLayers.LonLat(lonLat.lon, lonLat.lat).transform(
												map.getProjectionObject(), new OpenLayers.Projection('EPSG:4326'));
											options.on.complete.call(options.on.scope, {
												layer: {
													id: event.layer.mapItId,
													name: event.layer.name
												},
												id: event.mapItId,
												attributes: event.attributes,
												latitude: lonlatObject.lat,
												longitude: lonlatObject.lon
											});
										} 
									}
								});
								map.addControl(control);
							}
						}
					},
					
					addButton: function(icon) {
						var instance = this;
						
						var map = _map[instance.mapId];

						var controls = map.getControlsBy('mapItId', 'mainControlPanel')
						if (controls.length > 0) {
							var position = new OpenLayers.Control.Button({
							    displayClass: 'mapit-map-button mapit-map-originalPositionButton', 
							    trigger: function() {
									var latitude = instance.get('center.latitude');
									var longitude = instance.get('center.longitude');
									var zoom = instance.get('center.zoom');
									var lonlatObject = new OpenLayers.LonLat(longitude, latitude).transform(
										new OpenLayers.Projection('EPSG:4326'), map.getProjectionObject()
									);
							    	map.setCenter(new OpenLayers.LonLat(lonlatObject.lon, lonlatObject.lat), zoom);
							    }
							});
							
							var button = [];
							
							button.push(position);							
						}
					},
					
					activateDrag: function(layerName) {
						var instance = this;
						
						instance._enableDrag(layerName, false);
					},
					
					deactivateDrag: function(layerName) {
						var instance = this;
					
						instance._enableDrag(layerName, true);
					},
					
					register: function(event, scope, callback) {
						var instance = this;
						
						instance.on(event, function(event) {
							callback.call(scope, event.obj);
						});
					},
					
					refreshLayer: function(layerName) {
						var instance = this;
						
						var map = _map[instance.mapId];
						
						var layers = map.getLayersByName(layerName);
						if (layers.length > 0) {
							layers[0].refresh();
						}
					},
					
					showLayer: function(layerId) {
						var instance = this;
						
						var map = _map[instance.mapId];
						
						var layers = map.getLayersBy('mapItId', layerId);
						if (layers.length > 0) {
							layers[0].setVisibility(true);
						}
					},
					
					hideLayer: function(layerId) {
						var instance = this;
						
						var map = _map[instance.mapId];
						
						var layers = map.getLayersBy('mapItId', layerId);
						if (layers.length > 0) {
							layers[0].setVisibility(false);
							instance._unselectSelectedFeaturesByLayer(layers[0]);
						}
					},
					
					showLayerByName: function(layerName) {
						var instance = this;
						
						var map = _map[instance.mapId];
						
						var layers = map.getLayersByName(layerName);
						if (layers.length > 0) {
							layers[0].setVisibility(true);
						}
					},
					
					hideLayerByName: function(layerName) {
						var instance = this;
						
						var map = _map[instance.mapId];
						
						var layers = map.getLayersByName(layerName);
						if (layers.length > 0) {
							layers[0].setVisibility(false);
							instance._unselectSelectedFeaturesByLayer(layers[0]);
						}
					},
					
					unselectSelectedFeaturesByLayer: function(layerId) {
						var instance = this;
						
						var map = _map[instance.mapId];
						
						var layers = map.getLayersBy('mapItId', layerId);
						if (layers.length > 0) {
							instance._unselectSelectedFeaturesByLayer(layers[0]);
						}
					},
					
					getMapProjectionObject: function() {
						var instance = this;
						
						var map = _map[instance.mapId];
						
						return map.getProjectionObject();
					},
					
					getFeaturesByLayer: function(layerId) {
						var instance = this;
						
						return instance._getAllFeatures(layerId);
					},
					
					getAllFeatures: function() {
						var instance = this;
						
						return instance._getAllFeatures();
					},
					
					clickFeature: function(layerId, featureId) {
						var instance = this;
						
						var map = _map[instance.mapId];
						
						var layers = map.getLayersBy('mapItId', layerId);
						if (layers && layers.length > 0) {
							var feature = layers[0].getFeatureBy('mapItId', featureId);
							if (feature) {
								var selectControl = map.getControlsBy('mapItId', 'mapItSelectFeature')[0];
								selectControl.clickFeature(feature);
							}
						}
					},
					
					addMyPosition: function(longitude, latitude) {
						var instance = this;
						
						var map = _map[instance.mapId];
						
						instance.removeMyPosition();
						
						var myPositionLayer = new OpenLayers.Layer.Markers(
							Liferay.Language.get('label-mapitmap-my-position'), {
								mapItType: 'mapItPositionLayer',
								displayInLayerSwitcher: false
						});	
						
						var size = new OpenLayers.Size(21, 25);
			            var offset = new OpenLayers.Pixel(-(size.w/2), -size.h);
			            var icon = new OpenLayers.Icon(MapItMap.DEFAULT_MARKER, size, offset);
			            
			            var lonlatObject = new OpenLayers.LonLat(longitude, latitude).transform(
							new OpenLayers.Projection('EPSG:4326'), map.getProjectionObject()
						);
			            
			            myPositionLayer.addMarker(new OpenLayers.Marker(new OpenLayers.LonLat(
			            	lonlatObject.lon, lonlatObject.lat), icon));
						
						map.addLayer(myPositionLayer);
					},
					
					removeMyPosition: function() {
						var instance = this;
						
						var map = _map[instance.mapId];
						
						var layers = map.getLayersBy('mapItType', 'mapItPositionLayer');
						if (layers.length > 0) {
							layers[0].destroy();
						}
					},
					
					_enableDrag: function(layerName, disabled) {
						var instance = this;
						
						var map = _map[instance.mapId];
						
						var layers = map.getLayersByName(layerName);
						if (layers.length > 0) {	
							var controls = map.getControlsByClass(OpenLayers.Control.DragFeature.prototype.CLASS_NAME)
							for (var i=0; i<controls.length; i++) {
								if (controls[i] && controls[i].layer && controls[i].layer.name == layerName) {
									if (!disabled) {
										controls[i].activate();
									}
									else {
										controls[i].deactive();
									}
									break;
								}
							}
						}
					},
					
					_addLayer: function(layer, options) {
						var instance = this;
						
						var map = _map[instance.mapId];
						
						layer.events.register('visibilitychanged', layer, function(event) {
							var map = this.map;
							var popups = map.popups;
							for (var i = 0; i < popups.length && popups[i]; i++) {
								var popup = popups[i];
								var feature = popup.feature;
								if (feature && feature.layer && feature.layer.mapItId == this.mapItId) {
									if (this.mapItType == 'mapItSelectLayer') {
					                	var selectControl = map.getControlsBy('mapItId', 'mapItSelectFeature')[0];
					                	selectControl.unselect(feature);
					                } else { 
					                	instance.hidePopup({
											layerId: feature.layer.mapItId,
											featureId: feature.mapItId
										});
					                }
								}
							}
						});
						
						if (layer.protocol && layer.protocol.CLASS_NAME == OpenLayers.Protocol.HTTP.prototype.CLASS_NAME) {
							
							layer.events.register('loadend', layer, function(event) {
				    			var map = this.map;
				    			var extent = map.getExtent();
				    			var obj = map.mapItTmpMoveObject;
				    			var i = 0;
								while (i < obj.length) {
									if (obj[i].layerId == this.mapItId) {
										var feature = null;
										if (obj[i].feature.geometry && obj[i].feature.geometry.getBounds()) {
											if (extent.intersectsBounds(obj[i].feature.geometry.getBounds())) {
												for (var j = 0; j < this.features.length && feature == null; j++) {
													if (this.features[j].mapItId == obj[i].feature.mapItId) {
														feature = this.features[j];
													}
												}
											}
										}
										if (feature) {
											var popups = map.popups;
											for (var k = 0; k <popups.length && popups[k]; k++) {
												var popup = popups[k];
												if (popup.feature && popup.feature.mapItId == feature.mapItId) {
													feature.layer = this
													popup.hide();
													popup.feature = feature;
													feature.popup = popup;
													if (this.mapItType == 'mapItSelectLayer') {
														this.events.unregister('featureselected', instance, instance._featureSelected);
														var selectControl = map.getControlsBy('mapItId', 'mapItSelectFeature')[0];
														selectControl.select(feature);
														this.events.register('featureselected', instance, instance._featureSelected);
													}
													popup.show();
													break;
												}
											}
										}
										else {
											feature = obj[i].feature;
											feature.layer = this
											if (this.mapItType == 'mapItSelectLayer') {
												var selectControl = map.getControlsBy('mapItId', 'mapItSelectFeature')[0];
												selectControl.unselect(feature);
											} else {
												instance.hidePopup({
													layerId: feature.layer.mapItId,
													featureId: feature.mapItId
												});
											}
										}
										obj.splice(i, 1);
									}
									else {
										i++;
									}
								}
				            });
						} 
						else {
				    		
				    		map.events.register('moveend', layer, function(event) {
								var map = this.map;
							    var extent = map.getExtent();
							    var obj = map.mapItTmpMoveObject;
							    var i = 0;
								while (i < obj.length) {
									if (obj[i].layerId == this.mapItId) {
										var feature = null;
										if (obj[i].feature.geometry && obj[i].feature.geometry.getBounds()) {
											if (!extent.intersectsBounds(obj[i].feature.geometry.getBounds())) {
												for (var j = 0; j < this.features.length && feature == null; j++) {
													if (this.features[j].mapItId == obj[i].feature.mapItId) {
														feature = this.features[j];
													}
												}
											}
										}
										if (feature) {
											feature.layer = this;
											if (this.mapItType == 'mapItSelectLayer') {
												var selectControl = map.getControlsBy('mapItId', 'mapItSelectFeature')[0];
												selectControl.unselect(feature);
											} else {
												instance.hidePopup({
													layerId: feature.layer.mapItId,
													featureId: feature.mapItId
												});
											}
										}
								        obj.splice(i, 1);
									} else {
							        	i++;
							        }
								}
							});
				    	}
						
						var zoomToAllFeatures = false;
				    	
				    	if (options) {
				    		zoomToAllFeatures = (typeof options.zoomToAllFeatures != 'undefined' && options.zoomToAllFeatures);
				    	}
	
						layer.events.register('loadend', instance, function(event) {
							var layer = event.object;
							var loaded = layer.mapItLoaded;
				    		if (!loaded) {
				    			layer.mapItLoaded = true;
				    			// zoom to features at startup
				    			if (zoomToAllFeatures) {
				    				instance.zoomToDataExtent(layer.name);
				    			}
				    		}
				    		
				    		if (options && typeof options.on != 'undefined' && typeof options.on.loadend != 'undefined') {
				    			
				    			if (!(typeof options.on.scope != 'undefined')) {
				    				options.on.scope = window;
				    			}
				    			
				    			options.on.loadend.call(options.on.scope, {
				    				portletId: instance.get('portletId'),
				    				layer: {
				    					id: layer.mapItId,
					    				name: layer.name,
				    				}
				    			});
				    		}
				    		else {
				    			instance.fire('layerReady', {
				    				portletId: instance.get('portletId'),
				    				layer: {
				    					id: layer.mapItId,
					    				name: layer.name,
				    				}
				    			});
				    		}
			            });
						
						map.addLayer(layer);
						
						var selectable = true;
						
						if (options && typeof options.selectable != 'undefined') {
							selectable = options.selectable;
						}
									
				    	if (selectable) {
				    		
				    		layer.mapItType = 'mapItSelectLayer';
				    		
				    		layer.events.on({
							    'featureselected': instance._featureSelected,
							    'featureunselected': instance._featureUnselected,
							    scope: instance
							});
				    		
				    		if (options && typeof options.on != 'undefined'
				    			&& (typeof options.on.featureSelected != 'undefined' 
				    				|| typeof options.on.featureUnselected != 'undefined')) {
				    			
				    			if (!(typeof options.on.scope != 'undefined')) {
				    				options.on.scope = window;
				    			}
				    			
				    			instance.on(('featureselected_' + layer.mapItId), function(event) {
				    				options.on.featureSelected.call(options.on.scope, event.obj);
				    			});
				    			instance.on(('featureunselected_' + layer.mapItId), function(event) {
				    				options.on.featureUnselected.call(options.on.scope, event.obj);
				    			});
				    		}
				    		else {
				    			instance.on(('featureselected_' + layer.mapItId), function(event) {
				    				instance.fire('featureSelected', event.obj);
				    			});
				    			instance.on(('featureunselected_' + layer.mapItId), function(event) {
				    				instance.fire('featureUnselected', event.obj);
				    			});
				    		}
							
							var selectControl = map.getControlsBy('mapItId', 'mapItSelectFeature');
							
							if (selectControl.length == 0) {
								selectControl = new OpenLayers.Control.SelectFeature([layer], {
									mapItId: 'mapItSelectFeature'
								});
								map.addControl(selectControl);
								selectControl.activate();
							}
							else {
								var layers = map.getLayersBy('mapItType', 'mapItSelectLayer');
								selectControl[0].setLayer(layers);
							}
				    	}
				    	else {
				    		layer.mapItType = '';
				    	}
					},
					
					_loadMap: function() {
						var instance = this;
						
						var mapType = instance.get('mapType');
						var apiKey = instance.get('apiKey'); 
						var apiVersion = instance.get('apiVersion');
						
						var language = themeDisplay.getLanguageId().split('_');
						
						var map = _map[instance.mapId];
						
						if (mapType == MapItMap.MAP_TYPE.OPEN_STREET_MAP) {
							
							var osm = new OpenLayers.Layer.OSM(null, null, {
							    transitionEffect: 'resize'
							});
							
							osm.events.register('loadend', instance, function(event) {
								var layer = event.object;
								var loaded = layer.mapItLoaded;
					    		if (!loaded) {
					    			layer.mapItLoaded = true;
					    			this._addControls();
					    			this._ready();
					    		}
				            });
							
							map.addLayer(osm);
							
							instance._center();
						}
						else if (mapType == MapItMap.MAP_TYPE.GOOGLE_MAP) {
							
							var googleMapsJS = 'https://maps.googleapis.com/maps/api/js';
							
							var googleMaspsJSLoaded = !!A.one("script[src^='" + googleMapsJS + "'");
							
							if (googleMaspsJSLoaded) {
								A.one("script[src^='" + googleMapsJS + "'").on('loaded', function() {
									instance._loadGoogleMap();
								});
							} else {
								
								var googleMapsURL = Liferay.Util.addParams({
									v: apiVersion,
									key: apiKey,
									language: language[0],
									sensor: 'true'
								}, googleMapsJS);

								A.jsonp(googleMapsURL, function() {
									instance._loadGoogleMap();
									setTimeout(function() {
										A.one("script[src^='" + googleMapsJS + "'").fire('loaded');
									}, 1000);
								});
							}
						}
						else if (mapType == MapItMap.MAP_TYPE.BING_MAP) {
							
							var layer = null;
													
							if(!!apiKey) {
								
								var road = new OpenLayers.Layer.Bing({
						            name: Liferay.Language.get('label-mapitmap-road'),
						            key: apiKey,
						            type: 'Road',
						            numZoomLevels: 21,
						            transitionEffect: 'resize'
						        });
								
						        var hybrid = new OpenLayers.Layer.Bing({
						            name: Liferay.Language.get('label-mapitmap-hybrid'),
						            key: apiKey,
						            type: 'AerialWithLabels',
						            numZoomLevels: 21,
						            transitionEffect: 'resize'
						        });
						        
						        var aerial = new OpenLayers.Layer.Bing({
						            name: Liferay.Language.get('label-mapitmap-aerial'),
						            key: apiKey,
						            type: 'Aerial',
						            numZoomLevels: 21,
						            transitionEffect: 'resize'
						        });
						        
						        map.addLayers([road, hybrid, aerial]);
						        
						        layer = road;
							} else {
								
								var road = new OpenLayers.Layer.Bing({
						            name: Liferay.Language.get('label-mapitmap-road'),
						            type: 'Road',
						            numZoomLevels: 21,
						            transitionEffect: 'resize'
						        });
								
						        var hybrid = new OpenLayers.Layer.Bing({
						            name: Liferay.Language.get('label-mapitmap-hybrid'),
						            type: 'AerialWithLabels',
						            numZoomLevels: 21,
						            transitionEffect: 'resize'
						        });
						        
						        var aerial = new OpenLayers.Layer.Bing({
						            name: Liferay.Language.get('label-mapitmap-aerial'),
						            type: 'Aerial',
						            numZoomLevels: 21,
						            transitionEffect: 'resize'
						        });
						        
						        map.addLayers([road, hybrid, aerial]);
						        
						        layer = road;
							}
							
							layer.events.register('loadend', instance, function(event) {
								var layer = event.object;
								var loaded = layer.mapItLoaded;
					    		if (!loaded) {
					    			layer.mapItLoaded = true;
					    			this._addControls();
					    			this._ready();
					    		}
				            });
	
							instance._center();
						}
						else if (mapType == MapItMap.MAP_TYPE.BASE) {
							
//							var wms = new OpenLayers.Layer.WMS(
//								Liferay.Language.get('label-mapitmap-base'),
//					            'http://vmap0.tiles.osgeo.org/wms/vmap0', {
//									layers: 'basic', transparent: true, 
//									numZoomLevels: 21, transitionEffect: 'resize'
//								},
//					            {isBaseLayer: true, opacity: .7}
//					        ); 
							
							var wms = new OpenLayers.Layer.WMS(
									'Regione Toscana',
									//'http://www502.regione.toscana.it/wmsraster/com.rt.wms.RTmap/wms?map=wmsosm_d&map_resolution=91&', {
									'http://www502.regione.toscana.it/wmsraster/com.rt.wms.RTmap/wms?map=wmsosm_g&map_resolution=91&', {
										styles: 'default', 
										layers: 'default', transparent: true},
										{
											isBaseLayer: true,
//											minResolution: 0.000001341104507446289,
											minResolution: "auto",
							                maxResolution: 0.0054931640625,
										}
							);

//							var wms = new OpenLayers.Layer.WMS(
//									'Comuni',
//				                    'http://www502.regione.toscana.it/wmsraster/com.rt.wms.RTmap/wms?map=wmsambamm&map_resolution=91', {               
//				                        styles: 'contorno_senza_etichette', 
//				                        layers: 'rt_ambamm.idcomuni.rt.poly', transparent: true},
//				                    {isBaseLayer: true}
//								);					
							
							map.addLayer(wms);
							
							wms.events.register('loadend', instance, function(event) {
								var layer = event.object;
								var loaded = layer.mapItLoaded;
					    		if (!loaded) {
					    			layer.mapItLoaded = true;
					    			this._addControls();
					    			this._ready();
					    		}
				            });
	
							instance._center();
						}
						else if (mapType == MapItMap.MAP_TYPE.BASE_LABEL) {
							
							var wms = new OpenLayers.Layer.WMS(
								Liferay.Language.get('label-mapitmap-base'),
					            'http://vmap0.tiles.osgeo.org/wms/vmap0', {
									layers: 'basic,clabel,ctylabel,statelabel',
									transparent: true, numZoomLevels: 21,
									transitionEffect: 'resize'
								},
					            {isBaseLayer: true, opacity: .7}
						    ); 
							
							map.addLayer(wms);
							
							wms.events.register('loadend', instance, function(event) {
								var layer = event.object;
								var loaded = layer.mapItLoaded;
					    		if (!loaded) {
					    			layer.mapItLoaded = true;
					    			this._addControls();
					    			this._ready();
					    		}
				            });
	
							instance._center();
						}
					},
					
					_loadGoogleMap: function() {
						var instance = this;
						
						var map = _map[instance.mapId];
						
						var mapStyles = instance.get('mapStyles');
						
						var aerial = new OpenLayers.Layer.Google(
					        Liferay.Language.get('label-mapitmap-aerial'), {
					        numZoomLevels: 21, 
					        transitionEffect: 'resize'
					    });
						
						var terrain = new OpenLayers.Layer.Google(
							Liferay.Language.get('label-mapitmap-physical'), {
				            type: google.maps.MapTypeId.TERRAIN, 
				            numZoomLevels: 21, 
				            transitionEffect: 'resize'
				        });	
						
						var hybrid = new OpenLayers.Layer.Google(
							Liferay.Language.get('label-mapitmap-hybrid'), {
				            type: google.maps.MapTypeId.HYBRID, 
				            numZoomLevels: 21,
				            transitionEffect: 'resize'
				        });
						
						var satellite = new OpenLayers.Layer.Google(
				            Liferay.Language.get('label-mapitmap-satellite'), {
				            type: google.maps.MapTypeId.SATELLITE, 
				            numZoomLevels: 21,
				            transitionEffect: 'resize'
				        });
						
						map.addLayers([aerial, terrain, hybrid, satellite]);
						
						google.maps.event.addListener(aerial.mapObject, 'tilesloaded', function() {
							var loaded = this.mapItLoaded;
				    		if (!loaded) {
				    			this.mapItLoaded = true;
				    			instance._addControls();
								instance._ready();
				    		}
						});
						
						if (!!mapStyles) {
							aerial.mapObject.setOptions({styles: mapStyles});
						}

						instance._center();
					},
						
					_center: function() {
						var instance = this;
						
						var map = _map[instance.mapId];
						
						var latitude = instance.get('center.latitude');
						var longitude = instance.get('center.longitude');
						var zoom = instance.get('center.zoom');
						
						var geoLocation = instance.get('center.geoLocation');
						
						var saveLastPosition = instance.get('saveLastPosition');
	
						if (saveLastPosition) {
							var portletId = instance.get('portletId');
	
							map.events.register('moveend', map, function () {
							    instance._setCookie('MAPIT_X_' + portletId, map.getCenter().lon, 0, '/' );
							    instance._setCookie('MAPIT_Y_' + portletId, map.getCenter().lat, 0, '/' );
							    instance._setCookie('MAPIT_Z_' + portletId, map.getZoom(), 0, '/' );
							});
							
							map.events.register('changebaselayer', map, function () {
								instance._setCookie('MAPIT_BASE_' + portletId, map.baseLayer.name, 0, '/');
							});
							
							var base = instance._getCookie('MAPIT_BASE_' + portletId);
							var lon = instance._getCookie('MAPIT_X_' + portletId);
						   	var lat = instance._getCookie('MAPIT_Y_' + portletId);
						   	var z = instance._getCookie('MAPIT_Z_' + portletId);
	
						   	if (lon != null && lat != null && z != null) {
						    	map.setCenter(new OpenLayers.LonLat(lon, lat), z);
							} else {
								var lonlatObject = new OpenLayers.LonLat(longitude, latitude).transform(
									new OpenLayers.Projection('EPSG:4326'), map.getProjectionObject()
								);
								map.setCenter(new OpenLayers.LonLat(lonlatObject.lon, lonlatObject.lat), zoom);
						   	}
						   	
							if (base) {
								var layers = map.getLayersByName(base);
								if(layers != null && layers.length > 0) {
									map.setBaseLayer(map.getLayersByName(base)[0]);
								}
							}
							
						} else {
							if (geoLocation) {
								navigator.geolocation.getCurrentPosition(function(location) {
									instance.set('center.latitude', location.coords.latitude);
									instance.set('center.longitude', location.coords.longitude);
									var lonlatObject = new OpenLayers.LonLat(
											location.coords.longitude, location.coords.latitude).transform(
										new OpenLayers.Projection('EPSG:4326'), map.getProjectionObject()
									);
									map.setCenter(new OpenLayers.LonLat(lonlatObject.lon, lonlatObject.lat), zoom);
								}, function(err) {
									var lonlatObject = new OpenLayers.LonLat(longitude, latitude).transform(
										new OpenLayers.Projection('EPSG:4326'), map.getProjectionObject()
									);
									map.setCenter(new OpenLayers.LonLat(lonlatObject.lon, lonlatObject.lat), zoom);
								});
							} else {
								var lonlatObject = new OpenLayers.LonLat(longitude, latitude).transform(
									new OpenLayers.Projection('EPSG:4326'), map.getProjectionObject()
								);
								map.setCenter(new OpenLayers.LonLat(lonlatObject.lon, lonlatObject.lat), zoom);
							}
						}
					},
					
					_addControls: function() {
						var instance = this;
						
						var map = _map[instance.mapId];
						
						var id = instance.get('id');
						
						var portletId = instance.get('portletId');
						var namespace = Liferay.Util.getPortletNamespace(portletId);
						
						var zoomType = instance.get('controls.zoomType');
						
						if (!!zoomType) {
							if (zoomType == MapItMap.ZOOM_TYPE.PAN_ZOOM_BAR) {
								map.addControl(new OpenLayers.Control.PanZoomBar({
									displayClass: 'olControlZoom'
								}));
							} else if (zoomType == MapItMap.ZOOM_TYPE.PAN_ZOOM) {
								map.addControl(new OpenLayers.Control.PanZoom({
									displayClass: 'olControlZoom'
								}));
							} else if (zoomType == MapItMap.ZOOM_TYPE.ZOOM){
								map.addControl(new OpenLayers.Control.Zoom({
									displayClass: 'olControlZoom'
								}));
							}
						}
						
						if (instance.get('controls.overviewMap')) {
							var ovControl = new OpenLayers.Control.OverviewMap();
						    ovControl.isSuitableOverview = function() {
						        return false;
						    };
							map.addControl(ovControl);
						}
						
						map.addControl(new OpenLayers.Control.KeyboardDefaults());
						map.addControl(new OpenLayers.Control.Attribution());
						map.addControl(new OpenLayers.Control.Navigation());
						map.addControl(new OpenLayers.Control.TouchNavigation());
						
						if (instance.get('controls.scale')) {
							map.addControl(new OpenLayers.Control.ScaleLine());
						}
						
						if (instance.get('controls.layerSwitch')) {
							map.addControl(new OpenLayers.Control.LayerSwitcher({'ascending': true}));
						}
						
						if (instance.get('controls.mousePosition')) {
							map.addControl(new OpenLayers.Control.MousePosition({
								prefix: Liferay.Language.get('label-mapitmap-coordinates'),
								numDigits: 6,
								id: 'mapit-map-utm-mouse',
								displayProjection: new OpenLayers.Projection('EPSG:4326')
							}));
						}

						if (instance.get('controls.navToolbar')) {
							var NavToolbarCustom = OpenLayers.Class(OpenLayers.Control.NavToolbar, {
								initialize: function() { 
							          OpenLayers.Control.Panel.prototype.initialize.apply(this);
							          this.addControls([
							            new OpenLayers.Control.Navigation(),
							            new OpenLayers.Control.ZoomBox()  
							          ]);
								}
							});					
							map.addControl(new NavToolbarCustom());
						}
						
						var button = [];
						
						if (instance.get('center.geoLocation')) {
							var myPosition = new OpenLayers.Control.Button({
								displayClass: 'mapit-map-button mapit-map-myPositionButton', 
								type: OpenLayers.Control.TYPE_TOGGLE, 
								eventListeners: {
									'activate': function() { 
										navigator.geolocation.getCurrentPosition(function(location) {
											instance.addMyPosition(location.coords.longitude, location.coords.latitude);
										}, function(err) {
											var latitude = instance.get('center.latitude');
											var longitude = instance.get('center.longitude');
											var zoom = instance.get('center.zoom');
											var lonlatObject = new OpenLayers.LonLat(longitude, latitude).transform(
												new OpenLayers.Projection('EPSG:4326'), map.getProjectionObject()
											);
											map.setCenter(new OpenLayers.LonLat(lonlatObject.lon, lonlatObject.lat), zoom);
										});
									}, 
									'deactivate': function() { 
										instance.removeMyPosition();
									}
								}
							});	
							button.push(myPosition);
						}
						
						if(instance.get('controls.fullScreen')) {	
							var fullScreen = new OpenLayers.Control.Button({
								displayClass: 'mapit-map-button mapit-map-fullScreenButton', 
								type: OpenLayers.Control.TYPE_TOGGLE, 
								eventListeners: {
									'activate': function() { 
										A.one('#' + id).addClass('mapit-map-full-screen');
										A.one('#' + id).removeClass('mapit-map-no-full-screen');
										map.updateSize();
									}, 
									'deactivate': function() { 
										A.one('#' + id).removeClass('mapit-map-full-screen');
										A.one('#' + id).addClass('mapit-map-no-full-screen');
										map.updateSize();
									}
								}
							});	
							button.push(fullScreen);
						}
								
						if(instance.get('controls.home')) {	
							var position = new OpenLayers.Control.Button({
							    displayClass: 'mapit-map-button mapit-map-originalPositionButton', 
							    trigger: function() {
							    	var latitude = instance.get('center.latitude');
									var longitude = instance.get('center.longitude');
									var zoom = instance.get('center.zoom');
									var lonlatObject = new OpenLayers.LonLat(longitude, latitude).transform(
										new OpenLayers.Projection('EPSG:4326'), map.getProjectionObject()
									);
							    	map.setCenter(new OpenLayers.LonLat(lonlatObject.lon, lonlatObject.lat), zoom);
							    }
							});
							button.push(position);
						}
						
						var panelButton = new OpenLayers.Control.Panel({
							displayClass: 'mapit-map-controlPanel',
							mapItId: 'mainControlPanel'
						});
						
						map.addControl(panelButton);
						panelButton.addControls(button);
						
						// add events
						instance._click();
					},
					
					_ready: function() {
						var instance = this;
						
						var portletId = instance.get('portletId');
						
						instance.fire('ready', {portletId: portletId});
					},
					
					_featureSelected: function(event) {
						var instance = this;
				    	
				    	var map = _map[instance.mapId];
						
						var feature = event.feature;
						var layer = feature.layer;
			    		var lonLat = feature.geometry.getBounds().getCenterLonLat();
			    		
		    			var lonlatObject = new OpenLayers.LonLat(lonLat.lon, lonLat.lat).transform(
							map.getProjectionObject(), new OpenLayers.Projection('EPSG:4326'));	
		    				
		    			instance.fire(('featureselected_' + layer.mapItId), {
		    				obj: {
		    					portletId: instance.get('portletId'),
		    					feature: {
				    				id: feature.mapItId,
				    				layer: { 
				    					id: layer.mapItId,
				    				},
					    			latitude: lonlatObject.lat,
					    			longitude: lonlatObject.lon,
					    			attributes: feature.attributes
					    		}
		    				}
			    		});	
					},
					
					_featureUnselected: function(event) {
						var instance = this;
						
						var map = _map[instance.mapId];
						
						var feature = event.feature;
						var layer = feature.layer;
			    		var lonLat = feature.geometry.getBounds().getCenterLonLat();
			    		
		    			var lonlatObject = new OpenLayers.LonLat(lonLat.lon, lonLat.lat).transform(
							map.getProjectionObject(), new OpenLayers.Projection('EPSG:4326'));	
				    	
				    	instance.fire(('featureunselected_' + layer.mapItId), {
				    		obj: {
				    			portletId: instance.get('portletId'),
		    					feature: {
				    				id: feature.mapItId,
				    				layer: { 
				    					id: layer.mapItId,
				    				},
					    			latitude: lonlatObject.lat,
					    			longitude: lonlatObject.lon,
					    			attributes: feature.attributes
					    		}
		    				}
			    		});
					},
					
					_moveEvents: function() {
						var instance = this;
						
						var map = _map[instance.mapId];
						
						map.mapItTmpMoveObject = [];
						
						map.events.register('movestart', map, function(event) {
							var popups = this.popups;
							for (var i = 0; i < popups.length && popups[i]; i++) {
								var popup = popups[i];
								var feature = popup.feature;
								if (feature) {
									if (feature.layer) {
										var layer = feature.layer;
										if (!popup.mapItCloseOnMove) {
											var clone = feature.clone();
											clone.mapItId = feature.mapItId;
											this.mapItTmpMoveObject.push({
												feature: clone,
												layerId: layer.mapItId
											});
										}
										else {
											if (layer.mapItType == 'mapItSelectLayer') {
							                	var selectControl = map.getControlsBy('mapItId', 'mapItSelectFeature')[0];
							                	selectControl.unselect(feature);
							                } else { 
							                	instance.hidePopup({
													layerId: layer.mapItId,
													featureId: feature.mapItId
												});
							                }
										}
									}
									else {
										instance.hidePopup({
											popupId: popup.mapItId
										});
									}
								}
								else {
									if (popup.mapItCloseOnMove) {
										instance.hidePopup({
											popupId: popup.mapItId
										});
									}
								}
							}
						});
					},
					
					_click: function() {
						var instance = this;
						
						var map = _map[instance.mapId];
						
						var id = instance.get('id');
						
						A.one('#' + id).on('contextmenu', function(event) {
							event.preventDefault();
						});
						
						var portletId = instance.get('portletId');
						
						var clickControl = new OpenLayers.Control.MapItClick({
							trigger: function(event) {
								var map = event.object;
								var lonLat = map.getLonLatFromPixel(event.xy);
								var lonlatObject = new OpenLayers.LonLat(lonLat.lon, lonLat.lat).transform(
									map.getProjectionObject(), new OpenLayers.Projection('EPSG:4326')
								);
								instance.fire('rightclick', {
									obj: {
										portletId: portletId,
										latitude: lonlatObject.lat,
										longitude: lonlatObject.lon
									}
								});
							}
						});
		                map.addControl(clickControl);
		                clickControl.activate();
					},
					
					_resizePopup: function(iframeId, popupId) {
						var instance = this;
						
						var map = _map[instance.mapId];
						
						
						var iframe = document.getElementById(iframeId);
						var doc = (iframe.contentDocument) ? iframe.contentDocument : iframe.contentWindow.document;
						doc.body.setAttribute('style', 'padding:0px;margin:0px');
						
						var dimensionDiv = doc.getElementById('iframe-content');
						
						var formhr = doc.getElementById('hrefFm');
						formhr.setAttribute('style', 'margin:0px');
						
						var width = doc.body.clientWidth;
						if(dimensionDiv){
							var height = dimensionDiv.clientHeight;
						}else{
							var height = 200;
						}
						if (Liferay.Browser.isWebKit()) {
							/*TODO vedere se aggiustare qualcosa per Webkit*/
						}
						
						var popups = map.popups;
						for (var i=0; i<popups.length && popups[i]; i++) {
							if (popups[i].mapItId == popupId) {
								popups[i].updateSizeByDimension(width, height);
								popups[i].show();
								/*doc.documentElement.setAttribute('style','height:'+height+'px');*/
								A.one('#' + iframeId).set('width', '100%');
								/*A.one('#'+iframeId).set('height',height+'px');*/
								break;
							}
						}
					},
					
					_setCookie: function(name, value, expires, path, domain, secure) {
						
						var today = new Date();
						today.setTime(today.getTime());
	
						if (expires) {
							expires = expires * 1000 * 60 * 60 * 24;
						}
						
						var d = new Date(today.getTime() + (expires));
						
						var options = {
							path: path,
							domain: domain,
							expires: d.toGMTString(),
							secure: secure
						};
							
						A.Cookie.set(name, value, options);
					},
					
					_getCookie: function(name) {
						
						return A.Cookie.get(name);
					},
					
					_getAllFeatures: function(layerId) {
						var instance = this;
						
						var map = _map[instance.mapId];
						
						var layersMap = { layers: [] };
						
						var layers = [];
						
						if (typeof layerId != 'undefined') {
							layers = map.getLayersBy('mapItId', layerId);
						} else {
							layers = map.getLayersByClass('OpenLayers.Layer.Vector');
						}
						
						for (var i=0; i<layers.length; i++) {
							var layer = layers[i];
							var features = layer.features;
							var obj = { id: layer.mapItId, name: layer.name, features: [] };
							for (var j=0; j<features.length; j++) {
								var lonLat = features[j].geometry.getBounds().getCenterLonLat();
				    			var lonlatObject = new OpenLayers.LonLat(lonLat.lon, lonLat.lat).transform(
									map.getProjectionObject(), new OpenLayers.Projection('EPSG:4326'));	
								obj.features.push({
									id: features[j].mapItId,
									latitude: lonlatObject.lat,
					    			longitude: lonlatObject.lon,
									attributes: features[j].attributes
								});
							}
							layersMap.layers.push(obj);
						}
						
						return layersMap;
					},
					
					_unselectSelectedFeaturesByLayer: function(layer) {
						var instance = this;
						
						var map = _map[instance.mapId];
						
						if (layer.selectedFeatures.length > 0) {
							var features = layer.features;
							for (var j=0; j<features.length; j++) {
								var feature = features[j];
								if (layer.selectedFeatures.indexOf(feature) >= 0) {
									var selectControl = map.getControlsBy('mapItId', 'mapItSelectFeature')[0];
				                	selectControl.unselect(feature);
								}
							}
						}
					}
				}
			}
		);
		
		Liferay.MapItMap = MapItMap;
		
		/**
		 * Open Layers
		 */

		OpenLayers.Popup.MapItFramedCloud = OpenLayers.Class(OpenLayers.Popup.FramedCloud, {

		    /**
		     * Property: positionBlocks
		     * {Object} Hash of differen position blocks, keyed by relativePosition
		     *     two-character code string (ie "tl", "tr", "bl", "br")
		     */
		    positionBlocks: {
		    	 "tl": {
		             'offset': new OpenLayers.Pixel(44, 0),
		             'padding': new OpenLayers.Bounds(8, 40, 8, 9),
		             'blocks': [
		                 { // top-left
		                     size: new OpenLayers.Size('auto', 'auto'),
		                     anchor: new OpenLayers.Bounds(0, 51, 22, 0),
		                     position: new OpenLayers.Pixel(0, 0)
		                 },
		                 { //top-right
		                     size: new OpenLayers.Size(22, 'auto'),
		                     anchor: new OpenLayers.Bounds(null, 50, 0, 0),
		                     position: new OpenLayers.Pixel(-1238, 0)
		                 },
		                 { //bottom-left
		                     size: new OpenLayers.Size('auto', 19),
		                     anchor: new OpenLayers.Bounds(0, 32, 22, null),
		                     position: new OpenLayers.Pixel(0, -633)
		                 },
		                 { //bottom-right
		                     size: new OpenLayers.Size(22, 18),
		                     anchor: new OpenLayers.Bounds(null, 32, 0, null),
		                     position: new OpenLayers.Pixel(-1238, -634)
		                 },
		                 { // stem
		                     size: new OpenLayers.Size(81, 35),
		                     anchor: new OpenLayers.Bounds(null, 0, 0, null),
		                     position: new OpenLayers.Pixel(0, -708)
		                 }
		             ]
		         },
		         "tr": {
		             'offset': new OpenLayers.Pixel(-45, 0),
		             'padding': new OpenLayers.Bounds(8, 40, 8, 9),
		             'blocks': [
		                 { // top-left
		                     size: new OpenLayers.Size('auto', 'auto'),
		                     anchor: new OpenLayers.Bounds(0, 51, 22, 0),
		                     position: new OpenLayers.Pixel(0, 0)
		                 },
		                 { //top-right
		                     size: new OpenLayers.Size(22, 'auto'),
		                     anchor: new OpenLayers.Bounds(null, 50, 0, 0),
		                     position: new OpenLayers.Pixel(-1238, 0)
		                 },
		                 { //bottom-left
		                     size: new OpenLayers.Size('auto', 19),
		                     anchor: new OpenLayers.Bounds(0, 32, 22, null),
		                     position: new OpenLayers.Pixel(0, -633)
		                 },
		                 { //bottom-right
		                     size: new OpenLayers.Size(22, 19),
		                     anchor: new OpenLayers.Bounds(null, 32, 0, null),
		                     position: new OpenLayers.Pixel(-1238, -633)
		                 },
		                 { // stem
		                     size: new OpenLayers.Size(81, 35),
		                     anchor: new OpenLayers.Bounds(0, 0, null, null),
		                     position: new OpenLayers.Pixel(-215, -707)
		                 }
		             ]
		         },
		        "bl": {
		            'offset': new OpenLayers.Pixel(45, 0),
		            'padding': new OpenLayers.Bounds(8, 9, 8, 20),
		            'blocks': [
		                { // top-left
		                    size: new OpenLayers.Size('auto', 'auto'),
		                    anchor: new OpenLayers.Bounds(0, 21, 22, 12),
		                    position: new OpenLayers.Pixel(0, 0)
		                },
		                { //top-right
		                    size: new OpenLayers.Size(22, 'auto'),
		                    anchor: new OpenLayers.Bounds(null, 21, 0, 12),
		                    position: new OpenLayers.Pixel(-1238, 0)
		                },
		                { //bottom-left
		                    size: new OpenLayers.Size('auto', 21),
		                    anchor: new OpenLayers.Bounds(0, 0, 22, null),
		                    position: new OpenLayers.Pixel(0, -629)
		                },
		                { //bottom-right
		                    size: new OpenLayers.Size(22, 21),
		                    anchor: new OpenLayers.Bounds(null, 0, 0, null),
		                    position: new OpenLayers.Pixel(-1238, -629)
		                },
		                { // stem
		                    size: new OpenLayers.Size(81, 13),
		                    anchor: new OpenLayers.Bounds(null, null, 0, 0),
		                    position: new OpenLayers.Pixel(-101, -674)
		                }
		            ]
		        },
		        "br": {
		            'offset': new OpenLayers.Pixel(-44, 0),
		            'padding': new OpenLayers.Bounds(8, 9, 8, 20),
		            'blocks': [
		                { // top-left
		                    size: new OpenLayers.Size('auto', 'auto'),
		                    anchor: new OpenLayers.Bounds(0, 21, 22, 12),
		                    position: new OpenLayers.Pixel(0, 0)
		                },
		                { //top-right
		                    size: new OpenLayers.Size(22, 'auto'),
		                    anchor: new OpenLayers.Bounds(null, 21, 0, 12),
		                    position: new OpenLayers.Pixel(-1238, 0)
		                },
		                { //bottom-left
		                    size: new OpenLayers.Size('auto', 21),
		                    anchor: new OpenLayers.Bounds(0, 0, 22, null),
		                    position: new OpenLayers.Pixel(0, -629)
		                },
		                { //bottom-right
		                    size: new OpenLayers.Size(22, 21),
		                    anchor: new OpenLayers.Bounds(null, 0, 0, null),
		                    position: new OpenLayers.Pixel(-1238, -629)
		                },
		                { // stem
		                    size: new OpenLayers.Size(81, 13),
		                    anchor: new OpenLayers.Bounds(0, null, null, 0),
		                    position: new OpenLayers.Pixel(-311, -674)
		                }
		            ]
		        }
		    },
		    
		    /**
		     * APIMethod: updateSizeByDimension
		     */
		    updateSizeByDimension: function(width, height) {
		        
		        // determine actual render dimensions of the contents by putting its
		        // contents into a fake contentDiv (for the CSS) and then measuring it
		        var preparedHTML = "<div class='" + this.contentDisplayClass+ "'>" + 
		            this.contentDiv.innerHTML + 
		            "</div>";
		 
		        var containerElement = (this.map) ? this.map.div : document.body;
		        var realSize = new OpenLayers.Size(width, height);

		        // is the "real" size of the div is safe to display in our map?
		        var safeSize = this.getSafeContentSize(realSize);

		        var newSize = null;
		        if (safeSize.equals(realSize)) {
		            //real size of content is small enough to fit on the map, 
		            // so we use real size.
		            newSize = realSize;

		        } else {
		            // make a new 'size' object with the clipped dimensions 
		            // set or null if not clipped.
		            var fixedSize = {
		                w: (safeSize.w <= realSize.w) ? safeSize.w : null,
		                h: (safeSize.h <= realSize.h) ? safeSize.h : null
		            };
		        
		            if (fixedSize.w && fixedSize.h) {
		                //content is too big in both directions, so we will use 
		                // max popup size (safeSize), knowing well that it will 
		                // overflow both ways.                
		                newSize = safeSize;
		            } else {
		                //content is clipped in only one direction, so we need to 
		                // run getRenderedDimensions() again with a fixed dimension
		                var clippedSize = OpenLayers.Util.getRenderedDimensions(
		                    preparedHTML, fixedSize, {
		                        displayClass: this.contentDisplayClass,
		                        containerElement: containerElement
		                    }
		                );
		                
		                //if the clipped size is still the same as the safeSize, 
		                // that means that our content must be fixed in the 
		                // offending direction. If overflow is 'auto', this means 
		                // we are going to have a scrollbar for sure, so we must 
		                // adjust for that.
		                //
		                var currentOverflow = OpenLayers.Element.getStyle(
		                    this.contentDiv, "overflow"
		                );
		                if ( (currentOverflow != "hidden") && 
		                     (clippedSize.equals(safeSize)) ) {
		                    var scrollBar = OpenLayers.Util.getScrollbarWidth();
		                    if (fixedSize.w) {
		                        clippedSize.h += scrollBar;
		                    } else {
		                        clippedSize.w += scrollBar;
		                    }
		                }
		                
		                newSize = this.getSafeContentSize(clippedSize);
		            }
		        }                        
		        this.setSize(newSize);     
		    },

		    CLASS_NAME: 'OpenLayers.Popup.MapItFramedCloud'
		});

		OpenLayers.Control.MapItClick = OpenLayers.Class(OpenLayers.Control, {
			
			defaultHandlerOptions: {
				'single': true,
				'double': false,
				'pixelTolerance': 0,
				'stopSingle': false,
				'stopDouble': false
			},
			
			handleRightClicks: true,
			
			initialize: function(options) {
				this.handlerOptions = OpenLayers.Util.extend({}, this.defaultHandlerOptions);
				OpenLayers.Control.prototype.initialize.apply(this, arguments);
				this.handler = new OpenLayers.Handler.Click(this, {'rightclick': this.trigger}, this.handlerOptions);
			},
			
			CLASS_NAME: 'OpenLayers.Control.MapItClick'
		});

		OpenLayers.Format.MapItJSON = OpenLayers.Class(OpenLayers.Format.JSON, {

			
			/**
		     * APIProperty: defaultStyle
		     * defaultStyle allows one to control the default styling of the features.
		     *    It should be a symbolizer hash. By default, this is set to match the
		     *    Layer.Text behavior, which is to use the default OpenLayers Icon.
		     */
		    defaultStyle: null,
		     
		    /**
		     * APIProperty: extractStyles
		     * set to true to extract styles from the TSV files, using information
		     * from the image or icon, iconSize and iconOffset fields. This will result
		     * in features with a symbolizer (style) property set, using the
		     * default symbolizer specified in <defaultStyle>. Set to false if you
		     * wish to use a styleMap or OpenLayers.Style options to style your
		     * layer instead.
		     */
		    extractStyles: true,
		    
		    /**
		     * Constructor: OpenLayers.Format.MapItJSON
		     * Create a new parser for JSON.
		     *
		     * Parameters:
		     * options - {Object} An optional object whose properties will be set on
		     *     this instance.
		     */
		    initialize: function(options) {
		        options = options || {};

		        if(options.extractStyles !== false) {
		            options.defaultStyle = {
		                'externalGraphic': OpenLayers.Util.getImageLocation("marker.png"),
		                'graphicWidth': 21,
		                'graphicHeight': 25,
		                'graphicXOffset': -10.5,
		                'graphicYOffset': -12.5
		            };
		        }
		        
		        OpenLayers.Format.prototype.initialize.apply(this, [options]);
		    }, 

		    /**
		     * APIMethod: read
		     * Deserialize a MapItJSON string.
		     *
		     * Parameters:
		     * json - {String} A MapItJSON string
		     *
		     * Returns: 
		     * Array({<OpenLayers.Feature.Vector>})
		     */
		    read: function(json) {
		        var results = [];
		        var obj = null;
		        if (typeof json == "string") {
		            obj = OpenLayers.Format.JSON.prototype.read.apply(this, [json]);
		        } else { 
		            obj = json;
		        }    
		        if(!obj) {
		            OpenLayers.Console.error("Bad JSON: " + json);
		        } else if (!obj.features) {
		        	OpenLayers.Console.error("No features: " + json);
		        } else {
		            for(var i=0, len=obj.features.length; i<len; ++i) {
		                try {

		                	var object = obj.features[i];
		                	
		                	if (object.latitude && object.longitude && object.pk) {
		                		
		                		var geometry = new OpenLayers.Geometry.Point(
			        		    	parseFloat(object.longitude), parseFloat(object.latitude));

			    			    var attributes = {
			    					title: object.title,
			    					description: object.description,
			    					classNameId: object.classNameId,
			    					classPK: object.classPK
			    				}; 
			    			 
			    			    var styles = null;
			    			    
			    			    if (object.icon) {					    	
			    			    	styles = {
			    		    			externalGraphic: object.icon, 
			    						graphicHeight: 25, 
			    						graphicWidth: 21, 
			    						graphicXOffset:-12, 
			    						graphicYOffset:-25,
			    						cursor: 'pointer'
			    					};
			    			    }
		     
		    			    	if (this.internalProjection && this.externalProjection) {
		                            geometry.transform(this.externalProjection, this.internalProjection); 
		                        }
		    			    	
		    			    	var feature = new OpenLayers.Feature.Vector(geometry, attributes, styles);
		    			    	feature.mapItId = object.pk;
		    			    	results.push(feature);
		    			    }
		                } catch(err) {
		                    results = null;
		                    OpenLayers.Console.error(err);
		                }
		            }
		        }
		        return results;
		    },

		    CLASS_NAME: 'OpenLayers.Format.MapItJSON'
		}); 
	},
	'',
	{
		requires: ['aui-base', 'aui-base-lang', 'aui-component', 'aui-io-request', 'aui-loading-mask-deprecated', 'jsonp', 'jsonp-url', 'event-contextmenu', 'cookie', 'open-layers']
	}
);